LIBRARY IEEE;
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;

ENTITY banker IS
        Generic (n         :integer :=3);   --numPlayers
        PORT (Clk          : IN STD_LOGIC;
              Reset        : IN STD_LOGIC;
              playerId     : IN STD_LOGIC_VECTOR (n-1 downto 0); -- from
              Action       : IN STD_LOGIC_VECTOR (2*n-3 downto 0); --from scorer 
              TakeAction   : IN STD_LOGIC; 
              cgNewHand    : IN STD_LOGIC;         
              bet          : IN STD_LOGIC_VECTOR (11 DOWNTO 0);
 
              balance      : OUT STD_LOGIC_VECTOR (11 downto 0); --Max1024
              GameOver     : OUT STD_LOGIC; -- The whole game is over
              balancerdy   : OUT STD_LOGIC);

            
END banker;
              
ARCHITECTURE a OF banker IS
              
type matrix is array (integer range n-2 downto 0) of STD_LOGIC_VECTOR (11 downto 0);
signal  initialBank    : STD_LOGIC_VECTOR (11 downto 0);
signal  CheckPlayer    : STD_LOGIC_VECTOR (n-2 downto 0);
signal  End_Game       : STD_LOGIC; -- end of hand
signal  HP_Stop        : STD_LOGIC;
signal  bet_store      : matrix;
--signal  G              : STD_LOGIC;
signal  Count          : STD_LOGIC_VECTOR (6 DOWNTO 0);
signal  End_BJ         : STD_LOGIC;
signal  PL_Out         : STD_LOGIC_VECTOR (n-2 downto 0);
signal  bank           : matrix;
signal  betstore       : matrix;
signal  BetFlag        : STD_LOGIC;
signal  EndHand        : STD_LOGIC;


begin


---------------------------
-- Handling Total Scores --
---------------------------
-- player1
process (Clk, Reset, bet, TakeAction, cgNewHand,PlayerId)
begin
        
        if (Clk = '1' AND Clk'EVENT) then
        	if (Reset = '1') then
                	 initialBank <= "000001111111"; --127
	        	 for i in n-2 downto 0 loop
                		bank(i) <= initialBank;
			end loop;
		elsif (cgNewHand = '1') then 
			BetFlag <= '1';
	        end if;
                if BetFlag ='1' then	
	        	for i in n-2 downto 0 loop
	                	if PlayerId(i) = '1' then
			                betstore(i) <= bet;
                        	        balance <= bank(i);
		                end if;
		        end loop;
		        if PlayerId(n-2) = '1' then
				BetFlag <= '0';
                        end if;
	       end if;	

        if EndHand = '1' then
        	balancerdy <= '0';
         	EndHand <= '0';
      	end if;        
	if (TakeAction = '1') then                         
		for i in n-2 downto 0 loop
		    --case when player = Win
			if Action(2*i + 1 downto 2*i) = "11" then  -- blackjack
 	        		bank(i) <= ( betstore(i) + betstore(i)) + bank(i);
				 --case when player wins (no BJ)
			elsif 
			    	Action(2*i + 1 downto 2*i) = "10"    then -- win
				bank(i) <= ( betstore(i)) + bank(i);
				    --case when player = push (Tie)
			elsif 
			    	Action(2*i + 1 downto 2*i) = "01" then -- tie
			        bank(i) <= bank(i);
				--case when player = Lose
			elsif 
				Action(2*i + 1 downto 2*i) = "00" then -- lose
		        	if (bank(i) >= betstore(i)) then                     
				    	bank(i) <= bank(i) - betstore(i);
				else
				        bank(i) <= "000000000000";
				        PL_Out(i) <= '1';
			        end if;
	  	 	end if;
		end loop;
		balancerdy <= '1';
         	EndHand <= '1';
      	end if;
end if;
       
end process;




-------------------------------------------------------------------
----------------------------------------------
-- Set End_Game to high after each end game --
----------------------------------------------
process(Clk, Reset)
begin
        if (Reset = '1') then
                End_Game <= '0';
        elsif (Clk'EVENT and Clk = '1') then
                if (EndHand = '1') then
                        End_Game <= '1';
                else End_Game <= '0';
                end if;
        end if;
end process;

-- OUTPUT Game_Over for the Controller --

process(Clk, Reset, End_BJ)
begin
        if (Reset = '1') then
                GameOver <= '0';
	else
       -- elsif (Clk'EVENT and Clk = '1') then
                if (End_BJ = '1') then
                        GameOver <= '1';
                else GameOver <= '0';
                end if;
        end if;
end process;


-----------------------------------------------------------------
-- End after 100 Games or after all Players finish their Chips --
-- or any player has 1024 chips
-----------------------------------------------------------------
process (Clk, Reset, End_Game)
begin
        if (Reset = '1') then
                Count <= "0000000";
        elsif (Clk'EVENT and Clk = '1') then
                if (End_Game = '1') then
                        Count <= Count + "0000001";
                else Count <= Count;
                end if;
        end if;
end process;

process (Clk, Reset, PL_Out, Count)
variable MaxB :STD_LOGIC;
variable Plcheck: INTEGER;
begin
        if (Reset = '1') then
                End_BJ <= '0';
        elsif (Clk'EVENT and Clk = '1') then
	bankmax : for i in n-2 downto 0 loop
			if bank(i) >= "001111111111" then
				MaxB := '1';
				exit bankmax;
			end if;
		   end loop bankmax;

          -- for i in n-2 downto 0 loop
		for i in n-2 downto 0 loop
			if PL_Out(i) = '1' then
				Plcheck := PLcheck + 1;
			end if;
		end loop;		
                
		if (Count = "1100100") OR (Plcheck = n-2) OR (MaxB='1') then
			MaxB := '0';
                        End_BJ <= '1';
			Plcheck := 0;
                else End_BJ <= '0';
                end if;
          -- end loop;
        end if;

end process;

end a;
